/* POP3 Query - check how many mails are waiting at the server */
/* (C) Christian Michel, Sindelfingen, cmichel@ballonspass.de  */
Call RxFuncAdd "SockLoadFuncs", "rxsock", "SockLoadFuncs"
Call SockLoadFuncs "silent"

Call QueryAccount "mypop.domain.de", "myuserid", "mypassword", "myuserid@mydomain.com"
Exit


QueryAccount: Procedure
  Parse Arg POP3Server, POP3User, Password, AccountName

  mysock = Login(POP3Server, POP3User, Password)
  If mysock \= -1 Then
    Do
      Say AccountName || ":" GetMailCount(mysock) "mails waiting at server."
      Call Logout mysock
    End
  Return


Login: Procedure
  Parse Arg Server, Account, Password

  rc = SockGetHostByName(Server, "host.!")
  If rc = 0 Then
    Do
      Say "POP3 server name <" || Server || "> could not be resolved. Terminating!"
      Return -1
    End

  host.!family = "AF_INET"
  host.!port = 110

  Socket = SockSocket("AF_INET", "SOCK_STREAM", 0)
  If Socket = -1 Then
    Do
      Say "Could not create local socket. Terminating!"
      Return -1
    End

  rc = SockConnect(Socket, "host.!")
  If rc \= 0 Then
    Do
      Say "Could not connect to POP3 server <" || Server || ">. Terminating!"
      Return -1
    End

  Answer = ReadReplyLine(Socket)
  If Word(Answer, 1) = "-ERR" Then
    Do
      Say "Error connecting to POP3 server <" || Server || ">."
      Say "Error message was:" Answer
      Say "Terminating!"
      Return -1
    End

  Answer = SendCommand(Socket, "USER " || Account)
  If Word(Answer, 1) = "-ERR" Then
    Do
      Say "Unknown POP3 user <" || Account || ">."
      Say "Error message was:" Answer
      Say "Terminating!"
      Return -1
    End

  Answer = SendCommand(Socket, "PASS " || Password)
  If Word(Answer, 1) = "-ERR" Then
    Do
      Say "Invalid POP3 password."
      Say "Error message was:" Answer
      Say "Terminating!"
      Return -1
    End

  Return Socket


Logout: Procedure
  Parse Arg Socket
  Call SendCommand Socket, "QUIT"
  Call SockShutdown Socket, 2
  Call SockClose Socket
  Return


SendCommand: Procedure
  Parse Arg Socket, Command
  CRLF = "0D0A"x
  Call SockSend Socket, Command || CRLF
  Return ReadReplyLine(Socket)


GetMailCount: Procedure
  Parse Arg Socket
  Data = SendCommand(Socket, "STAT")
  If Word(Data, 1) = "+OK" Then
    Parse Value Data With "+OK" NoOfMails NoBytes
  Else
    Do
      Say "Could not retrieve number of mails waiting."
      Say "Error was:" Data
      NoOfMails = 0
    End
  Return NoOfMails


GetMail: Procedure
  Parse Arg Socket, MsgNumber

  Command = "RETR" MsgNumber
  Call SendCommand Socket, Command

  Say "Reading message" MsgNumber || ":"
  Message = ""
  Data = ""
  Do Forever
    Bytes = SockRecv(Socket, "Data", 1024)
    Message = Message . Data
    If Bytes >= 3 & SubStr(Data, Bytes-2, 3) = "." || "0D0A"x Then
      Leave
  End

  Return Message


ReadReplyLine: Procedure
  Parse Arg Socket

  Line = ""
  Data = ""
  Bytes = 1
  Do While (Pos("0A"x, Line) = 0) & (Bytes \= 0)
    If SockRecv(Socket, "Data", 1024, "MSG_PEEK") = 0 Then
      Leave
    Bytes = SockRecv(Socket, "Data", 1024)
    Line = Line || Data
  End

  If Length(Line) > 0 Then
    Do
      If Length(SubStr(Line, Pos("0A"x, Line) + 1)) > 0 Then
        Say "Warning, multi line reply received! Additional lines ignored."

      Reply = Strip(Left(Line, Pos("0A"x, Line) - 1), "B", "0D"x)
    End
  Else
    Reply = "-ERR - no answer from POP3 server"

  Return Reply
